@@ -1,9 +1,11 @@
-use ruma_identifiers::{EventId, RoomVersionId, ServerNameBox};
+use ruma_identifiers::{EventId, RoomVersionId, ServerName};
+use ruma_serde::Base64DecodeError;
 use thiserror::Error;
 
 /// `ruma-signature`'s error type, wraps a number of other error types.
 #[derive(Debug, Error)]
 #[non_exhaustive]
+#[allow(clippy::enum_variant_names)]
 pub enum Error {
     /// [`JsonError`] wrapper.
     #[error("JSON error: {0}")]
@@ -21,9 +23,17 @@ pub enum Error {
     #[error("DER Parse error: {0}")]
     DerParse(pkcs8::der::Error),
 
-    /// [`SplitError`] wrapper.
-    #[error("Split error: {0}")]
-    SplitError(#[from] SplitError),
+    /// The signature's ID does not have exactly two components separated by a colon.
+    #[error("malformed signature ID: expected exactly 2 segment separated by a colon, found {0}")]
+    InvalidLength(usize),
+
+    /// The signature's ID contains invalid characters in its version.
+    #[error("malformed signature ID: expected version to contain only characters in the character set `[a-zA-Z0-9_]`, found `{0}`")]
+    InvalidVersion(String),
+
+    /// The signature uses an unsupported algorithm.
+    #[error("signature uses an unsupported algorithm: {0}")]
+    UnsupportedAlgorithm(String),
 
     /// PDU was too large
     #[error("PDU is larger than maximum of 65535 bytes")]
@@ -135,11 +145,11 @@ pub enum JsonType {
 pub enum VerificationError {
     /// For when a signature cannot be found for a `target`.
     #[error("Could not find signatures for {0:?}")]
-    SignatureNotFound(ServerNameBox),
+    SignatureNotFound(Box<ServerName>),
 
     /// For when a public key cannot be found for a `target`.
     #[error("Could not find public key for {0:?}")]
-    PublicKeyNotFound(ServerNameBox),
+    PublicKeyNotFound(Box<ServerName>),
 
     /// For when no public key matches the signature given.
     #[error("Not signed with any of the given public keys")]
@@ -151,11 +161,11 @@ pub enum VerificationError {
 }
 
 impl VerificationError {
-    pub(crate) fn signature_not_found<T: Into<ServerNameBox>>(target: T) -> Error {
+    pub(crate) fn signature_not_found<T: Into<Box<ServerName>>>(target: T) -> Error {
         Self::SignatureNotFound(target.into()).into()
     }
 
-    pub(crate) fn public_key_not_found<T: Into<ServerNameBox>>(target: T) -> Error {
+    pub(crate) fn public_key_not_found<T: Into<Box<ServerName>>>(target: T) -> Error {
         Self::PublicKeyNotFound(target.into()).into()
     }
 }
@@ -175,7 +185,7 @@ pub enum ParseError {
     /// For when an event ID, coupled with a specific room version, doesn't have a server name
     /// embedded.
     #[error("Event Id {0:?} should have a server name for the given room version {1:?}")]
-    ServerNameFromEventIdByRoomVersion(EventId, RoomVersionId),
+    ServerNameFromEventIdByRoomVersion(Box<EventId>, RoomVersionId),
 
     /// For when the extracted/"parsed" public key from a PKCS#8 v2 document doesn't match the
     /// public key derived from it's private key.
@@ -219,7 +229,7 @@ pub enum ParseError {
         string: String,
         /// The originating error.
         #[source]
-        source: base64::DecodeError,
+        source: Base64DecodeError,
     },
 }
 
@@ -228,7 +238,7 @@ impl ParseError {
         event_id: &EventId,
         room_version: &RoomVersionId,
     ) -> Error {
-        Self::ServerNameFromEventIdByRoomVersion(event_id.clone(), room_version.clone()).into()
+        Self::ServerNameFromEventIdByRoomVersion(event_id.to_owned(), room_version.clone()).into()
     }
 
     pub(crate) fn derived_vs_parsed_mismatch<P: Into<Vec<u8>>, D: Into<Vec<u8>>>(
@@ -245,25 +255,8 @@ impl ParseError {
     pub(crate) fn base64<T1: Into<String>, T2: Into<String>>(
         of_type: T1,
         string: T2,
-        source: base64::DecodeError,
+        source: Base64DecodeError,
     ) -> Error {
         Self::Base64 { of_type: of_type.into(), string: string.into(), source }.into()
     }
 }
-
-/// An error when trying to extract the algorithm and version from a key identifier.
-#[derive(Error, Debug)]
-#[non_exhaustive]
-pub enum SplitError {
-    /// The signature's ID does not have exactly two components separated by a colon.
-    #[error("malformed signature ID: expected exactly 2 segment separated by a colon, found {0}")]
-    InvalidLength(usize),
-
-    /// The signature's ID contains invalid characters in its version.
-    #[error("malformed signature ID: expected version to contain only characters in the character set `[a-zA-Z0-9_]`, found `{0}`")]
-    InvalidVersion(String),
-
-    /// The signature uses an unsupported algorithm.
-    #[error("unsupported algorithm: {0}")]
-    UnsupportedAlgorithm(String),
-}
